package meerkat.bulletinboard.sqlserver;

import java.util.List;

import com.google.protobuf.ProtocolStringList;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import meerkat.bulletinboard.BulletinBoardServer;
import meerkat.bulletinboard.MessageFilter;
import meerkat.comm.CommunicationException;
import meerkat.protobuf.Crypto.SignatureVerificationKey;
import meerkat.protobuf.Voting.BulletinBoardMessage;
import meerkat.crypto.Digest;
import meerkat.crypto.concrete.SHA256Digest;
import meerkat.protobuf.Voting.MessageID;

public abstract class BulletinBoardSQLServer implements BulletinBoardServer{
	
	protected Connection connection;
	
	protected Digest digest;
	
	protected List<SignatureVerificationKey> trusteeSignatureVerificationArray;
	protected int minTrusteeSignatures;

	protected List<List<SignatureVerificationKey>> pollingCommitteeSignatureVerificationKeyArray;
	protected int minCommiteeSignatures;
	
	/**
	 * This method initializes the signatures but does not implement the DB connection.
	 * Any full (non-abstract) extension of this class should
	 * 1. Establish a DB connection, and
	 * 2. Call this procedure
	 */
	@Override
	public void init() throws CommunicationException {
		// TODO write signature reading part.
		
		digest = new SHA256Digest();
	}

	/**
	 * This method verifies the authenticity of the received message based on
	 * the stored signatures.
	 * 
	 * @param msg
	 *            is the message to authenticate (containing the signature).
	 * @return TRUE if the message is authenticated and FALSE otherwise.
	 */
	private boolean verifyMessage(BulletinBoardMessage msg) {
		return true;
	}
	
	/**
	 * This procedure makes sure that all tags in the given list have an entry in the tags list. 
	 * @param tagIterator
	 */
	protected abstract void insertNewTags(String[] tags) throws CommunicationException;
	
	@Override
	public boolean postMessage(BulletinBoardMessage msg) throws CommunicationException {
		if (!verifyMessage(msg)) {
			return false;
		}
		
		PreparedStatement pstmt;
		String sql;
		byte[] msgID;
		
		ProtocolStringList tagList;
		String[] tags;
		
		digest.reset();
		digest.update(msg);
		
		msgID = digest.digest();
		
		try {
			
			sql = "INSERT INTO MsgTable (Id, Data, Signature) VALUES(?,?,?)";
		    pstmt = connection.prepareStatement(sql);
		    pstmt.setBytes(1, msgID);
		    pstmt.setBytes(2,msg.getMsg().getData().toByteArray());
		    pstmt.setBytes(3,msg.getSig().toByteArray());
		    pstmt.executeUpdate();
		    pstmt.close();
		    
		    tagList = msg.getMsg().getTagsList();
		    tags = new String[tagList.size()];
		    
		    insertNewTags(tags);
		    
		    sql = "INSERT INTO MsgTagTable (MsgId, TagId) SELECT (?, TagId) FROM TagTable WHERE tag=?";
	    	pstmt = connection.prepareStatement(sql);
		    
		    for (String tag : tags){
		    	pstmt.setBytes(1, msgID);
		    	pstmt.setString(2, tag);
		    	pstmt.addBatch();
		    }
		    
		    pstmt.executeBatch();
		    pstmt.close();
			
		} catch (SQLException e) {
			throw new CommunicationException("Error accessing DB: " + e.getMessage());
		}

		return true;
	}

	@Override
	public List<BulletinBoardMessage> readMessages(MessageFilter filter, int max) {
		// TODO Auto-generated method stub
		return null;
	}
	
	public void testPrint(){
		
		try {
	    
		    Statement statement = connection.createStatement();
			ResultSet rs = statement.executeQuery("select * from MsgTable");
			while (rs.next()) {
				// read the result set
				System.out.println("data = " + rs.getString("Data"));
				System.out.println("id = " + rs.getInt("Id"));
			}
			
			rs = statement.executeQuery("select * from MsgTagTable");
			while (rs.next()) {
				// read the result set
				System.out.println("MsgId = " + rs.getInt("MsgId"));
				System.out.println("TagId = " + rs.getString("TagId"));
			}
		} catch(SQLException e){
			System.out.println("Error reading from DB");
		}
		
	}

}
