package meerkat.bulletinboard.sqlserver;

import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

import meerkat.bulletinboard.MessageFilter;
import meerkat.bulletinboard.sqlserver.BulletinBoardSQLServer;
import meerkat.comm.CommunicationException;
import meerkat.protobuf.Voting.BulletinBoardMessage;


public class SQLiteBulletinBoardServer extends BulletinBoardSQLServer {

	protected static final int TIMEOUT = 20;
	
	/**
	 * This procedure initializes:
	 * 1. The database connection
	 * 2. The database tables (if they do not yet exist).
	 * 3. 
	 */
	@Override
	public void init() throws CommunicationException {
		
		try{
			
			connection = DriverManager.getConnection("jdbc:sqlite:./local-instances/meerkat.db");
			Statement statement = connection.createStatement();
			statement.setQueryTimeout(TIMEOUT);
	
			statement.executeUpdate("CREATE TABLE IF NOT EXISTS MsgTable (EntryNum INTEGER, Id varbinary(1000) UNIQUE, Data varbinary(1000), Signature varbinary(1000), PRIMARY KEY (EntryNum ASC))");
			statement.executeUpdate("CREATE TABLE IF NOT EXISTS TagTable (Id int, Tag v	archar(50) UNIQUE, PRIMARY KEY (Id ASC))");
			statement.executeUpdate("CREATE TABLE IF NOT EXISTS MsgTagTable (MsgId varbinary(1000), TagId int, FOREIGN KEY (MsgId) REFERENCES MsgTable(Id), FOREIGN KEY (TagId) REFERENCES TagTable(Id))");
			
			statement.close();
			
			super.init();
				
		} catch (SQLException e) {
			
			throw new CommunicationException("Couldn't form a connection with the database");
			
		}
				
	}


	@Override
	public List<BulletinBoardMessage> readMessages(MessageFilter filter, int max) {
		// TODO Auto-generated method stub
		return null;
	}
	
	public void close() throws CommunicationException{
		
		try{
			connection.close();
		} catch (SQLException e) {
			
			throw new CommunicationException("Couldn't close connection to the database");
			
		}
		
	}


	@Override
	protected void insertNewTags(String[] tags) throws CommunicationException {
		
		PreparedStatement pstmt;
		String sql;
		
		try {
			
		    sql = "INSERT OR IGNORE INTO TagTable(Tag) VALUES (?)";
	    	pstmt = connection.prepareStatement(sql);
		    
		    for (String tag : tags){
		    	pstmt.setString(1, tag);
		    	pstmt.addBatch();
		    }
		    
		    pstmt.executeBatch();
		    pstmt.close();
		    
		} catch (SQLException e){
			throw new CommunicationException("Error adding new tags to table");
		}
		
	}

}
