package meerkat.bulletinboard;

import java.util.List;

import meerkat.comm.CommunicationException;
import meerkat.protobuf.Voting.BulletinBoardMessage;

/**
 * Created by Arbel on 07/11/15.
 * 
 * This interface refers to a single instance of a Bulletin Board.
 * An implementation of this interface may use any DB and be hosted on any machine. 
 */

public interface BulletinBoardServer{
	
	/**
	 * This method initializes the server by reading the signature data and storing it.
	 * It also establishes the connection to the DB.
	 * @throws CommunicationException on DB connection error.
	 */
	public void init() throws CommunicationException;
	
	/**
     * Post a message to bulletin board.
     * @param msg is the actual (signed) message
	 * @return TRUE if the message has been authenticated and FALSE otherwise.
	 * @throws CommunicationException on DB connection error.
     */
	 public boolean postMessage(BulletinBoardMessage msg) throws CommunicationException;
	 
	 /**
	  * Read all messages posted matching the given filter.
	  * @param filter return only messages that match the filter (null means no filtering).
	  * @param max maximum number of messages to return (0=no limit)
	  * @return
	  */
	 List<BulletinBoardMessage> readMessages(MessageFilter filter, int max);
	 
	 /**
	 * This method closes the connection to the DB.
	 * @throws CommunicationException on DB connection error.
	 */
	 public void close() throws CommunicationException;
}
